@@ -20,8 +20,11 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE."""
 from array import array
 from itertools import chain, islice
 
+from pyproj import CRS, Proj
 from pyproj._transformer import _Transformer
 from pyproj.compat import cstrencode
+from pyproj.enums import TransformDirection, WktVersion
+from pyproj.exceptions import ProjError
 from pyproj.utils import _convertback, _copytobuffer
 
 try:
@@ -39,54 +42,119 @@ class Transformer(object):
     Additionally, it provides multiple methods for initialization.
     """
 
-    @staticmethod
-    def from_proj(proj_from, proj_to, skip_equivalent=False):
-        """Make a Transformer from a :obj:`pyproj.Proj` or input used to create one.
-
-        Parameters
-        ----------
-        proj_from: :obj:`pyproj.Proj` or input used to create one
-            Projection of input data.
-        proj_from: :obj:`pyproj.Proj` or input used to create one
-            Projection of output data.
-        skip_equivalent: bool, optional
-            If true, will skip the transformation operation if input and output 
-            projections are equivalent. Default is false.
-
-        Returns
-        -------
-        :obj:`pyproj.Transformer`
+    def __init__(self, base_transformer=None):
+        if not isinstance(base_transformer, _Transformer):
+            ProjError.clear()
+            raise ProjError(
+                "Transformer must be initialized using: "
+                "'from_crs', 'from_pipeline', or 'from_proj'."
+            )
+        self._transformer = base_transformer
 
+    @property
+    def name(self):
         """
+        str: Name of the projection.
+        """
+        return self._transformer.id
 
-        transformer = Transformer()
-        transformer._transformer = _Transformer.from_proj(
-            proj_from, proj_to, skip_equivalent
-        )
-        return transformer
+    @property
+    def description(self):
+        """
+        str: Description of the projection.
+        """
+        return self._transformer.description
+
+    @property
+    def definition(self):
+        """
+        str: Definition of the projection.
+        """
+        return self._transformer.definition
+
+    @property
+    def has_inverse(self):
+        """
+        bool: True if an inverse mapping exists.
+        """
+        return self._transformer.has_inverse
+
+    @property
+    def accuracy(self):
+        """
+        float: Expected accuracy of the transformation. -1 if unknown.
+        """
+        return self._transformer.accuracy
 
     @staticmethod
-    def from_crs(crs_from, crs_to, skip_equivalent=False):
-        """Make a Transformer from a :obj:`pyproj.CRS` or input used to create one.
+    def from_proj(proj_from, proj_to, skip_equivalent=False, always_xy=False):
+        """Make a Transformer from a :obj:`~pyproj.proj.Proj` or input used to create one.
 
         Parameters
         ----------
-        proj_from: :obj:`pyproj.CRS` or input used to create one
+        proj_from: :obj:`~pyproj.proj.Proj` or input used to create one
             Projection of input data.
-        proj_from: :obj:`pyproj.CRS` or input used to create one
+        proj_to: :obj:`~pyproj.proj.Proj` or input used to create one
             Projection of output data.
         skip_equivalent: bool, optional
             If true, will skip the transformation operation if input and output
             projections are equivalent. Default is false.
+        always_xy: bool, optional
+            If true, the transform method will accept as input and return as output
+            coordinates using the traditional GIS order, that is longitude, latitude
+            for geographic CRS and easting, northing for most projected CRS.
+            Default is false.
 
         Returns
         -------
-        :obj:`pyproj.Transformer`
+        :obj:`~Transformer`
 
         """
-        transformer = Transformer()
-        transformer._transformer = _Transformer.from_crs(
-            crs_from, crs_to, skip_equivalent
+        if not isinstance(proj_from, Proj):
+            proj_from = Proj(proj_from)
+        if not isinstance(proj_to, Proj):
+            proj_to = Proj(proj_to)
+
+        return Transformer(
+            _Transformer.from_crs(
+                proj_from.crs,
+                proj_to.crs,
+                skip_equivalent=skip_equivalent,
+                always_xy=always_xy,
+            )
+        )
+
+    @staticmethod
+    def from_crs(crs_from, crs_to, skip_equivalent=False, always_xy=False):
+        """Make a Transformer from a :obj:`~pyproj.crs.CRS` or input used to create one.
+
+        Parameters
+        ----------
+        crs_from: ~pyproj.crs.CRS or input used to create one
+            Projection of input data.
+        crs_to: ~pyproj.crs.CRS or input used to create one
+            Projection of output data.
+        skip_equivalent: bool, optional
+            If true, will skip the transformation operation if input and output
+            projections are equivalent. Default is false.
+        always_xy: bool, optional
+            If true, the transform method will accept as input and return as output
+            coordinates using the traditional GIS order, that is longitude, latitude
+            for geographic CRS and easting, northing for most projected CRS.
+            Default is false.
+
+        Returns
+        -------
+        :obj:`~Transformer`
+
+        """
+        transformer = Transformer(
+            _Transformer.from_crs(
+                CRS.from_user_input(crs_from),
+                CRS.from_user_input(crs_to),
+                skip_equivalent=skip_equivalent,
+                always_xy=always_xy,
+            )
         )
         return transformer
 
@@ -94,7 +162,7 @@ class Transformer(object):
     def from_pipeline(proj_pipeline):
         """Make a Transformer from a PROJ pipeline string.
 
-        https://proj4.org/operations/pipeline.html
+        https://proj.org/operations/pipeline.html
 
         Parameters
         ----------
@@ -103,14 +171,21 @@ class Transformer(object):
 
         Returns
         -------
-        :obj:`pyproj.Transformer`
+        ~Transformer
 
         """
-        transformer = Transformer()
-        transformer._transformer = _Transformer.from_pipeline(cstrencode(proj_pipeline))
-        return transformer
+        return Transformer(_Transformer.from_pipeline(cstrencode(proj_pipeline)))
 
-    def transform(self, xx, yy, zz=None, radians=False, errcheck=False):
+    def transform(
+        self,
+        xx,
+        yy,
+        zz=None,
+        tt=None,
+        radians=False,
+        errcheck=False,
+        direction=TransformDirection.FORWARD,
+    ):
         """
         Transform points between two coordinate systems.
 
@@ -122,14 +197,19 @@ class Transformer(object):
             Input y coordinate(s).
         zz: scalar or array (numpy or python), optional
             Input z coordinate(s).
+        tt: scalar or array (numpy or python), optional
+            Input time coordinate(s).
         radians: boolean, optional
             If True, will expect input data to be in radians and will return radians
             if the projection is geographic. Default is False (degrees). Ignored for
             pipeline transformations.
         errcheck: boolean, optional (default False)
             If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation 
+            By default errcheck=False and an invalid transformation
             returns ``inf`` and no exception is raised.
+        direction: ~pyproj.enums.TransformDirection, optional
+            The direction of the transform.
+            Default is :attr:`~pyproj.enums.TransformDirection.FORWARD`.
 
 
         Example:
@@ -139,20 +219,34 @@ class Transformer(object):
         >>> x3, y3 = transformer.transform(33, 98)
         >>> "%.3f  %.3f" % (x3, y3)
         '10909310.098  3895303.963'
-        >>> pipeline_str = "+proj=pipeline +step +proj=longlat +ellps=WGS84 +step +proj=unitconvert +xy_in=rad +xy_out=deg"
+        >>> pipeline_str = (
+        ...     "+proj=pipeline +step +proj=longlat +ellps=WGS84 "
+        ...     "+step +proj=unitconvert +xy_in=rad +xy_out=deg"
+        ... )
         >>> pipe_trans = Transformer.from_pipeline(pipeline_str)
         >>> xt, yt = pipe_trans.transform(2.1, 0.001)
         >>> "%.3f  %.3f" % (xt, yt)
         '120.321  0.057'
-        >>> transproj = Transformer.from_proj({"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'}, '+init=EPSG:4326')
-        >>> xpj, ypj, zpj = transproj.transform(-2704026.010, -4253051.810, 3895878.820, radians=True)
+        >>> transproj = Transformer.from_crs(
+        ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ...     '+init=EPSG:4326'
+        ... )
+        >>> xpj, ypj, zpj = transproj.transform(
+        ...     -2704026.010,
+        ...     -4253051.810,
+        ...     3895878.820,
+        ...     radians=True,
+        ... )
         >>> "%.3f %.3f %.3f" % (xpj, ypj, zpj)
         '-2.137 0.661 -20.531'
-        >>> transprojr = Transformer.from_proj('+init=EPSG:4326', {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'})
+        >>> transprojr = Transformer.from_crs(
+        ...     '+init=EPSG:4326',
+        ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ... )
         >>> xpjr, ypjr, zpjr = transprojr.transform(xpj, ypj, zpj, radians=True)
         >>> "%.3f %.3f %.3f" % (xpjr, ypjr, zpjr)
         '-2704026.010 -4253051.810 3895878.820'
-        >>> transformer = Transformer.from_crs("epsg:4326", 4326, skip_equivalent=True)
+        >>> transformer = Transformer.from_proj("epsg:4326", 4326, skip_equivalent=True)
         >>> xeq, yeq = transformer.transform(33, 98)
         >>> "%.0f  %.0f" % (xeq, yeq)
         '33  98'
@@ -165,18 +259,39 @@ class Transformer(object):
             inz, zisfloat, zislist, zistuple = _copytobuffer(zz)
         else:
             inz = None
+        if tt is not None:
+            intime, tisfloat, tislist, tistuple = _copytobuffer(tt)
+        else:
+            intime = None
         # call pj_transform.  inx,iny,inz buffers modified in place.
-        self._transformer._transform(inx, iny, inz, radians, errcheck=errcheck)
+        self._transformer._transform(
+            inx,
+            iny,
+            inz=inz,
+            intime=intime,
+            direction=direction,
+            radians=radians,
+            errcheck=errcheck,
+        )
         # if inputs were lists, tuples or floats, convert back.
         outx = _convertback(xisfloat, xislist, xistuple, inx)
         outy = _convertback(yisfloat, yislist, xistuple, iny)
+        return_data = (outx, outy)
         if inz is not None:
-            outz = _convertback(zisfloat, zislist, zistuple, inz)
-            return outx, outy, outz
-        else:
-            return outx, outy
+            return_data += (_convertback(zisfloat, zislist, zistuple, inz),)
+        if intime is not None:
+            return_data += (_convertback(tisfloat, tislist, tistuple, intime),)
+        return return_data
 
-    def itransform(self, points, switch=False, radians=False, errcheck=False):
+    def itransform(
+        self,
+        points,
+        switch=False,
+        time_3rd=False,
+        radians=False,
+        errcheck=False,
+        direction=TransformDirection.FORWARD,
+    ):
         """
         Iterator/generator version of the function pyproj.Transformer.transform.
 
@@ -186,16 +301,21 @@ class Transformer(object):
         points: list
             List of point tuples.
         switch: boolean, optional
-            If True x, y or lon,lat coordinates of points are switched to y, x 
+            If True x, y or lon,lat coordinates of points are switched to y, x
             or lat, lon. Default is False.
+        time_3rd: boolean, optional
+            If the input coordinates are 3 dimensional and the 3rd dimension is time.
         radians: boolean, optional
             If True, will expect input data to be in radians and will return radians
             if the projection is geographic. Default is False (degrees). Ignored for
             pipeline transformations.
         errcheck: boolean, optional (default False)
             If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation 
+            By default errcheck=False and an invalid transformation
             returns ``inf`` and no exception is raised.
+        direction: ~pyproj.enums.TransformDirection, optional
+            The direction of the transform.
+            Default is :attr:`~pyproj.enums.TransformDirection.FORWARD`.
 
 
         Example:
@@ -207,18 +327,41 @@ class Transformer(object):
         '2221638.801 2637034.372'
         '2212924.125 2619851.898'
         '2238294.779 2703763.736'
-        >>> pipeline_str = "+proj=pipeline +step +proj=longlat +ellps=WGS84 +step +proj=unitconvert +xy_in=rad +xy_out=deg"
+        >>> pipeline_str = (
+        ...     "+proj=pipeline +step +proj=longlat +ellps=WGS84 "
+        ...     "+step +proj=unitconvert +xy_in=rad +xy_out=deg"
+        ... )
         >>> pipe_trans = Transformer.from_pipeline(pipeline_str)
-        >>> for pt in pipe_trans.itransform([(2.1, 0.001)]): '{:.3f} {:.3f}'.format(*pt)
+        >>> for pt in pipe_trans.itransform([(2.1, 0.001)]):
+        ...     '{:.3f} {:.3f}'.format(*pt)
         '120.321 0.057'
-        >>> transproj = Transformer.from_proj({"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'}, '+init=EPSG:4326')
-        >>> for pt in transproj.itransform([(-2704026.010, -4253051.810, 3895878.820)], radians=True): '{:.3f} {:.3f} {:.3f}'.format(*pt)
+        >>> transproj = Transformer.from_crs(
+        ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ...     '+init=EPSG:4326',
+        ... )
+        >>> for pt in transproj.itransform(
+        ...     [(-2704026.010, -4253051.810, 3895878.820)],
+        ...     radians=True,
+        ... ):
+        ...     '{:.3f} {:.3f} {:.3f}'.format(*pt)
         '-2.137 0.661 -20.531'
-        >>> transprojr = Transformer.from_proj('+init=EPSG:4326', {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'})
-        >>> for pt in transprojr.itransform([(-2.137, 0.661, -20.531)], radians=True): '{:.3f} {:.3f} {:.3f}'.format(*pt)
+        >>> transprojr = Transformer.from_proj(
+        ...     '+init=EPSG:4326',
+        ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ... )
+        >>> for pt in transprojr.itransform(
+        ...     [(-2.137, 0.661, -20.531)],
+        ...     radians=True
+        ... ):
+        ...     '{:.3f} {:.3f} {:.3f}'.format(*pt)
         '-2704214.394 -4254414.478 3894270.731'
-        >>> transproj_eq = Transformer.from_proj('+init=EPSG:4326', 4326, skip_equivalent=True)
-        >>> for pt in transproj_eq.itransform([(-2.137, 0.661)]): '{:.3f} {:.3f}'.format(*pt)
+        >>> transproj_eq = Transformer.from_proj(
+        ...     '+init=EPSG:4326',
+        ...     '+proj=longlat +datum=WGS84 +no_defs +type=crs',
+        ...     skip_equivalent=True
+        ... )
+        >>> for pt in transproj_eq.itransform([(-2.137, 0.661)]):
+        ...     '{:.3f} {:.3f}'.format(*pt)
         '-2.137 0.661'
 
         """
@@ -230,29 +373,85 @@ class Transformer(object):
             raise ValueError("iterable must contain at least one point")
 
         stride = len(fst_pt)
-        if stride not in (2, 3):
-            raise ValueError("points can contain up to 3 coordinates")
+        if stride not in (2, 3, 4):
+            raise ValueError("points can contain up to 4 coordinates")
+
+        if time_3rd and stride != 3:
+            raise ValueError("'time_3rd' is only valid for 3 coordinates.")
 
         # create a coordinate sequence generator etc. x1,y1,z1,x2,y2,z2,....
         # chain so the generator returns the first point that was already acquired
         coord_gen = chain(fst_pt, (coords[c] for coords in it for c in range(stride)))
 
         while True:
-            # create a temporary buffer storage for the next 64 points (64*stride*8 bytes)
+            # create a temporary buffer storage for
+            # the next 64 points (64*stride*8 bytes)
             buff = array("d", islice(coord_gen, 0, 64 * stride))
             if len(buff) == 0:
                 break
 
             self._transformer._transform_sequence(
-                stride, buff, switch, radians, errcheck=errcheck
+                stride,
+                buff,
+                switch=switch,
+                direction=direction,
+                time_3rd=time_3rd,
+                radians=radians,
+                errcheck=errcheck,
             )
 
             for pt in zip(*([iter(buff)] * stride)):
                 yield pt
 
+    def to_wkt(self, version=WktVersion.WKT2_2018, pretty=False):
+        """
+        Convert the projection to a WKT string.
+
+        Version options:
+          - WKT2_2015
+          - WKT2_2015_SIMPLIFIED
+          - WKT2_2018
+          - WKT2_2018_SIMPLIFIED
+          - WKT1_GDAL
+          - WKT1_ESRI
+
+
+        Parameters
+        ----------
+        version: ~pyproj.enums.WktVersion
+            The version of the WKT output.
+            Default is :attr:`~pyproj.enums.WktVersion.WKT2_2018`.
+        pretty: bool
+            If True, it will set the output to be a multiline string. Defaults to False.
+
+        Returns
+        -------
+        str: The WKT string.
+        """
+        return self._transformer.to_wkt(version=version, pretty=pretty)
+
+    def __str__(self):
+        return self.definition
+
+    def __repr__(self):
+        return ("<{type_name}: {name}>\n" "{description}").format(
+            type_name=self._transformer.type_name,
+            name=self.name,
+            description=self.description,
+        )
+
 
 def transform(
-    p1, p2, x, y, z=None, radians=False, errcheck=False, skip_equivalent=False
+    p1,
+    p2,
+    x,
+    y,
+    z=None,
+    tt=None,
+    radians=False,
+    errcheck=False,
+    skip_equivalent=False,
+    always_xy=False,
 ):
     """
     x2, y2, z2 = transform(p1, p2, x1, y1, z1)
@@ -264,17 +463,20 @@ def transform(
     transformed to x2,y2,z2 in the coordinate system defined by p2.
 
     z1 is optional, if it is not set it is assumed to be zero (and
-    only x2 and y2 are returned). If the optional keyword 
-    'radians' is True (default is False), then all input and 
-    output coordinates will be in radians instead of the default 
+    only x2 and y2 are returned). If the optional keyword
+    'radians' is True (default is False), then all input and
+    output coordinates will be in radians instead of the default
     of degrees for geographic input/output projections.
-    If the optional keyword 'errcheck' is set to True an 
+    If the optional keyword 'errcheck' is set to True an
     exception is raised if the transformation is
     invalid. By default errcheck=False and ``inf`` is returned for an
     invalid transformation (and no exception is raised).
-    If the optional kwarg skip_equivalent is true (default is False), 
-    it will skip the transformation operation if input and output 
-    projections are equivalent.
+    If the optional kwarg skip_equivalent is true (default is False),
+    it will skip the transformation operation if input and output
+    projections are equivalent. If `always_xy` is toggled, the
+    transform method will accept as input and return as output
+    coordinates using the traditional GIS order, that is longitude, latitude
+    for geographic CRS and easting, northing for most projected CRS.
 
     In addition to converting between cartographic and geographic
     projection coordinates, this function can take care of datum
@@ -289,7 +491,7 @@ def transform(
 
     Example usage:
 
-    >>> from pyproj import Proj, transform 
+    >>> from pyproj import Proj, transform
     >>> # projection 1: UTM zone 15, grs80 ellipse, NAD83 datum
     >>> # (defined by epsg code 26915)
     >>> p1 = Proj(init='epsg:26915', preserve_units=False)
@@ -334,22 +536,25 @@ def transform(
     >>> x2, y2 = transform(c1, c2, x1, y1)
     >>> "%s  %s" % (str(x2)[:9],str(y2)[:9])
     '1402291.0  5076289.5'
-    >>> pj = Proj(init="epsg:4214")
-    >>> pjx, pjy = pj(116.366, 39.867)
-    >>> xr, yr = transform(pj, Proj(4326), pjx, pjy, radians=True, errcheck=True)
-    >>> "%.3f %.3f" % (xr, yr)
-    '0.696 2.031'
     >>> xeq, yeq = transform(4326, 4326, 30, 60, skip_equivalent=True)
     >>> "%.0f %.0f" % (xeq, yeq)
     '30 60'
     """
-    return Transformer.from_proj(p1, p2, skip_equivalent=skip_equivalent).transform(
-        x, y, z, radians, errcheck=errcheck
-    )
+    return Transformer.from_proj(
+        p1, p2, skip_equivalent=skip_equivalent, always_xy=always_xy
+    ).transform(xx=x, yy=y, zz=z, tt=tt, radians=radians, errcheck=errcheck)
 
 
 def itransform(
-    p1, p2, points, switch=False, radians=False, errcheck=False, skip_equivalent=False
+    p1,
+    p2,
+    points,
+    switch=False,
+    time_3rd=False,
+    radians=False,
+    errcheck=False,
+    skip_equivalent=False,
+    always_xy=False,
 ):
     """
     points2 = itransform(p1, p2, points1)
@@ -368,22 +573,26 @@ def itransform(
         - a Nx3 or Nx2 2d numpy array where N is the point number
         - a generator of coordinates (xi,yi) for 2d points or (xi,yi,zi) for 3d
 
-    If optional keyword 'switch' is True (default is False) then x, y or lon,lat coordinates
-    of points are switched to y, x or lat, lon. If the optional keyword 'radians' is True
-    (default is False), then all input and output coordinates will be in radians instead
+    If optional keyword 'switch' is True (default is False) then x, y or lon,lat
+    coordinates of points are switched to y, x or lat, lon.
+    If the optional keyword 'radians' is True (default is False),
+    then all input and output coordinates will be in radians instead
     of the default of degrees for geographic input/output projections.
-    If the optional keyword 'errcheck' is set to True an 
+    If the optional keyword 'errcheck' is set to True an
     exception is raised if the transformation is
     invalid. By default errcheck=False and ``inf`` is returned for an
     invalid transformation (and no exception is raised).
-    If the optional kwarg skip_equivalent is true (default is False), 
-    it will skip the transformation operation if input and output 
-    projections are equivalent.
+    If the optional kwarg skip_equivalent is true (default is False),
+    it will skip the transformation operation if input and output
+    projections are equivalent. If `always_xy` is toggled, the
+    transform method will accept as input and return as output
+    coordinates using the traditional GIS order, that is longitude, latitude
+    for geographic CRS and easting, northing for most projected CRS.
 
 
     Example usage:
 
-    >>> from pyproj import Proj, itransform 
+    >>> from pyproj import Proj, itransform
     >>> # projection 1: WGS84
     >>> # (defined by epsg code 4326)
     >>> p1 = Proj(init='epsg:4326', preserve_units=False)
@@ -396,14 +605,13 @@ def itransform(
     '411050.470 4497928.574'
     '399060.236 4486978.710'
     '458553.243 4523045.485'
-    >>> pj = Proj(init="epsg:4214")
-    >>> pjx, pjy = pj(116.366, 39.867)
-    >>> for pt in itransform(pj, Proj(4326), [(pjx, pjy)], radians=True, errcheck=True): '{:.3f} {:.3f}'.format(*pt)
-    '0.696 2.031'
-    >>> for pt in itransform(4326, 4326, [(30, 60)], skip_equivalent=True): '{:.0f} {:.0f}'.format(*pt)
+    >>> for pt in itransform(4326, 4326, [(30, 60)], skip_equivalent=True):
+    ...     '{:.0f} {:.0f}'.format(*pt)
     '30 60'
 
     """
-    return Transformer.from_proj(p1, p2, skip_equivalent=skip_equivalent).itransform(
-        points, switch, radians, errcheck=errcheck
+    return Transformer.from_proj(
+        p1, p2, skip_equivalent=skip_equivalent, always_xy=always_xy
+    ).itransform(
+        points, switch=switch, time_3rd=time_3rd, radians=radians, errcheck=errcheck
     )
