@@ -1,9 +1,10 @@
 import numpy as np
 import pytest
-from numpy.testing import assert_almost_equal, assert_equal
+from numpy.testing import assert_almost_equal
 
 import pyproj
 from pyproj import Proj, Transformer, itransform, transform
+from pyproj.enums import TransformDirection
 from pyproj.exceptions import ProjError
 
 
@@ -35,13 +36,10 @@ def test_illegal_transformation():
     )
     assert np.all(np.isinf(xx))
     assert np.all(np.isinf(yy))
-    try:
-        xx, yy = pyproj.transform(
+    with pytest.raises(ProjError):
+        pyproj.transform(
             p1, p2, (-180, -180, 180, 180, -180), (-90, 90, 90, -90, -90), errcheck=True
         )
-        assert_equal(None, "Should throw an exception when errcheck=True")
-    except ProjError:
-        pass
 
 
 def test_lambert_conformal_transform():
@@ -228,18 +226,28 @@ def test_itransform_time_3rd_invalid():
         list(itransform(7789, 8401, [(3496737.2679, 743254.4507)], time_3rd=True))
 
 
-def test_transform_error():
+def test_transform_no_error():
     pj = Proj(init="epsg:4555")
     pjx, pjy = pj(116.366, 39.867)
-    with pytest.raises(ProjError):
-        transform(pj, Proj(4326), pjx, pjy, radians=True, errcheck=True)
+    transform(pj, Proj(4326), pjx, pjy, radians=True, errcheck=True)
 
 
-def test_itransform_error():
+def test_itransform_no_error():
     pj = Proj(init="epsg:4555")
     pjx, pjy = pj(116.366, 39.867)
-    with pytest.raises(ProjError):
-        list(itransform(pj, Proj(4326), [(pjx, pjy)], radians=True, errcheck=True))
+    list(itransform(pj, Proj(4326), [(pjx, pjy)], radians=True, errcheck=True))
+
+
+def test_transform_no_exception():
+    # issue 249
+    transformer = Transformer.from_proj("+init=epsg:4326", "+init=epsg:27700")
+    transformer.transform(1.716073972, 52.658007833, errcheck=True)
+    transformer.itransform([(1.716073972, 52.658007833)], errcheck=True)
+
+
+def test_transform__out_of_bounds():
+    transformer = Transformer.from_proj("+init=epsg:4326", "+init=epsg:27700")
+    assert np.all(np.isinf(transformer.transform(100000, 100000, errcheck=True)))
 
 
 def test_transform_radians():
@@ -288,3 +296,122 @@ def test_itransform_radians():
         ),
         [(-2704026.010, -4253051.810, 3895878.820)],
     )
+
+
+def test_4d_transform__inverse():
+    transformer = Transformer.from_pipeline("+init=ITRF2008:ITRF2000")
+    assert_almost_equal(
+        transformer.transform(
+            xx=3513638.1999428216,
+            yy=778956.4532640711,
+            zz=5248216.453456361,
+            tt=2008.75,
+            direction=TransformDirection.INVERSE,
+        ),
+        (3513638.19380, 778956.45250, 5248216.46900, 2008.75),
+    )
+
+
+def test_transform_direction():
+    forward_transformer = Transformer.from_crs(4326, 3857)
+    inverse_transformer = Transformer.from_crs(3857, 4326)
+    assert inverse_transformer.transform(
+        -33, 24, direction=TransformDirection.INVERSE
+    ) == forward_transformer.transform(-33, 24)
+    ident_transformer = Transformer.from_crs(4326, 3857)
+    ident_transformer.transform(-33, 24, direction=TransformDirection.IDENT) == (
+        -33,
+        24,
+    )
+
+
+def test_always_xy__transformer():
+    transformer = Transformer.from_crs(2193, 4326, always_xy=True)
+    assert_almost_equal(
+        transformer.transform(1625350, 5504853),
+        (173.29964730317386, -40.60674802693758),
+    )
+
+
+def test_always_xy__transform():
+    assert_almost_equal(
+        transform(2193, 4326, 1625350, 5504853, always_xy=True),
+        (173.29964730317386, -40.60674802693758),
+    )
+
+
+def test_always_xy__itransform():
+    assert_almost_equal(
+        list(itransform(2193, 4326, [(1625350, 5504853)], always_xy=True)),
+        [(173.29964730317386, -40.60674802693758)],
+    )
+
+
+def test_transform_direction__string():
+    forward_transformer = Transformer.from_crs(4326, 3857)
+    inverse_transformer = Transformer.from_crs(3857, 4326)
+    assert inverse_transformer.transform(
+        -33, 24, direction="INVERSE"
+    ) == forward_transformer.transform(-33, 24, direction="FORWARD")
+    ident_transformer = Transformer.from_crs(4326, 3857)
+    ident_transformer.transform(-33, 24, direction="IDENT") == (-33, 24)
+
+
+def test_transform_direction__string_lowercase():
+    forward_transformer = Transformer.from_crs(4326, 3857)
+    inverse_transformer = Transformer.from_crs(3857, 4326)
+    assert inverse_transformer.transform(
+        -33, 24, direction="inverse"
+    ) == forward_transformer.transform(-33, 24, direction="forward")
+    ident_transformer = Transformer.from_crs(4326, 3857)
+    ident_transformer.transform(-33, 24, direction="ident") == (-33, 24)
+
+
+def test_transform_direction__invalid():
+    transformer = Transformer.from_crs(4326, 3857)
+    with pytest.raises(ValueError, match="Invalid value"):
+        transformer.transform(-33, 24, direction="WHEREVER")
+
+
+def test_from_pipeline__non_transform_input():
+    with pytest.raises(ProjError, match="Input is not a transformation"):
+        Transformer.from_pipeline("epsg:4326")
+
+
+def test_non_supported_initialization():
+    with pytest.raises(ProjError, match="Transformer must be initialized using"):
+        Transformer()
+
+
+def test_pj_info_properties():
+    transformer = Transformer.from_crs(4326, 3857)
+    assert transformer.name == "pipeline"
+    assert transformer.description == "Popular Visualisation Pseudo-Mercator"
+    assert transformer.definition.startswith("proj=pipeline")
+    assert transformer.has_inverse
+    assert transformer.accuracy == 0
+
+
+def test_to_wkt():
+    transformer = Transformer.from_crs(4326, 3857)
+    assert transformer.to_wkt().startswith(
+        'CONVERSION["Popular Visualisation Pseudo-Mercator"'
+    )
+
+
+def test_str():
+    assert str(Transformer.from_crs(4326, 3857)).startswith("proj=pipeline")
+
+
+def test_repr():
+    assert repr(Transformer.from_crs(7789, 8401)) == (
+        "<Transformation Transformer: helmert>\n" "ITRF2014 to ETRF2014 (1)"
+    )
+
+    assert repr(Transformer.from_crs(4326, 3857)) == (
+        "<Conversion Transformer: pipeline>\n" "Popular Visualisation Pseudo-Mercator"
+    )
+
+    assert repr(Transformer.from_crs(4326, 26917)) == (
+        "<Unknown Transformer: unknown>\n" "unavailable until proj_trans is called"
+    )
